/* src/admin_cmds.c - Diverses commandes pour admins
 * Copyright (C) 2002-2003 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: admin_cmds.c,v 1.53 2003/12/28 22:23:12 romexzf Exp $
 */

#include "main.h"
#include "outils.h"
#include "cs_cmds.h"
#include "fichiers.h"
#include "add_info.h"
#include "del_info.h"
#include "hash.h"
#include "config.h"
#include "divers.h"
#include "showcommands.h"
#include "aide.h"
#ifdef SQLLOG
#include <mysql.h>
#endif
#ifdef USE_WELCOMESERV
#include "welcome.h"
#endif
#include "web2cs.h"

int inviteme(aNick *nick, aChan *chaninfo, int parc, char **parv)
{ 	/* invite des admins sur le salon d'infos */
    putserv("%s " TOKEN_INVITE " %s :%s", cs.num, nick->nick, bot.pchan);
    return 1;
}

void CleanUp(void)
{
	aChan *c, *ct;
	aBan *b, *bt;
	anUser *u, *ut;
	anAccess *a, *at;
#ifdef USE_MEMOSERV
	aMemo *m, *mt;
#endif
	struct badchans *bc = badchead, *bct;
	struct badnicks *bn = badnhead, *bnt;
	struct ignore *ii = ignorehead, *it;
	aKill *k = killhead, *kt;
	int i = 0;

	free(quit_msg);
	free(mailprog);
	free(pasdeperm);
	free(network);

#ifdef SQLLOG
	if(bot.sql_id) mysql_close(bot.sql_id);
#endif

#ifdef TESTBOT
	w2c_closesocks();
#endif

	purge_nickandserv();

	for(;i < CHANHASHSIZE;i++) for(c = chan_tab[i];c;c = c->next)
	{
		ct = c->next;
		for(b = c->banhead;b;free(b->raison), free(b), b = bt) bt = b->next;
		if(c->suspend) free(c->suspend);
		if(c->motd) free(c->motd);
		free(c);
	}
	for(i = 0;i < USERHASHSIZE;i++) for(u = user_tab[i];u;u = ut)
	{
		ut = u->next;
		for(a = u->accesshead;a;free(a), a = at)
		{
			at = a->next;
			if(a->info) free(a->info);
		}
#ifdef USE_MEMOSERV
		for(m = u->memohead;m;free(m), m = mt) mt = m->next;
#endif
		if(u->suspend) free(u->suspend);
		if(u->lastlogin) free(u->lastlogin);
		free(u);
	}
	for(;k;free(k), k = kt) kt = k->next;
	for(;ii;free(ii), ii = it) it = ii->next;
	for(;bc;free(bc), bc = bct) bct = bc->next;
	for(;bn;free(bn), bn = bnt) bnt = bn->next;
	return;
}

int die(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	char *r = parc ? parv2msg(parc, parv, 1, 450) : quit_msg;

	want_to_quit = 1;
	write_cmds();

	putserv("%s " TOKEN_QUIT " :%s (par \2%s\2)", cs.num, r, nick->nick);

#ifdef USE_WELCOMESERV
	putserv("%s " TOKEN_QUIT " :%s (par \2%s\2)", ws.num, r, nick->nick);
#endif

	putserv("%s " TOKEN_SQUIT " %s 0 :%s (par \2%s\2)", bot.servnum, bot.server, r, nick->user->nick);
	remove(SCODERZ_PID);
	CleanUp();
	return 1;
}

/*
 * restart_bot parv[1->parc-1] = reason
 */
int restart_bot(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	FILE *fuptime;
	char *r = parc ? parv2msg(parc, parv, 1, 450) : quit_msg;

	if((fuptime = fopen("uptime.tmp", "w"))) /* permet de maintenir l'uptime (/me tricheur)*/
	{
		fprintf(fuptime, "%lu %d", bot.uptime, lastresetparse);
		fclose(fuptime);
	}

	want_to_quit = 1;
	write_cmds();

	putserv("%s " TOKEN_QUIT " :%s [\2RESTART\2] (par \2%s\2)", cs.num, r, nick->nick);
#ifdef USE_WELCOMESERV
	putserv("%s " TOKEN_QUIT " :%s [\2RESTART\2] (par \2%s\2)", ws.num, r, nick->nick);
#endif

	putserv("%s " TOKEN_SQUIT " %s 0 :%s [\2RESTART\2] (par \2%s\2)", bot.servnum, bot.server, r, nick->user->nick);

	CleanUp();
	remove(SCODERZ_PID);
	system("./services -h");
	return 1;
}

int cs_wall(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	aNick *n;
	int i = 0;
	char *buf = parv2msg(parc, parv, 1, 400);

	for(;i < NICKHASHSIZE;i++) for(n = nick_tab[i];n;n = n->next)
		if(IsAnAdmin(n->user)) csntc(n, "CSWall de \2%s\2: %s", nick->nick, buf);
	return 1;
}

int chcomname(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	const char *newcmd = parv[2], *lastcmd = parv[1];
	int i, icmd;

	if(strlen(newcmd) > 14)
		return csntc(nick, "Un nom de commande ne doit pas dpasser 14 caractres.");

	if((icmd = GetCurCmdIndexbyName(lastcmd)) < 0 || cs_cmds[icmd].cmd[0] == '\1')
		return csntc(nick, GetReply(nick, L_NOSUCHCMD), lastcmd);

	if((i = GetCurCmdIndexbyName(newcmd)) >= 0)
		return csntc(nick, "La commande %s existe dj.", newcmd);

	strcpy(cs_cmds[icmd].cmd, newcmd);
	csntc(nick, "La commande\2 %s\2 s'appelle maintenant\2 %s\2.", lastcmd, cs_cmds[icmd].cmd);
	BuildCommandsTable();
	write_cmds();
	return 1;
}

int chlevel(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	const char *cmd = parv[1];
	int level, i, adm = getoption("-admin", parv, parc, 2, -1);

	if((i = GetCurCmdIndexbyName(cmd)) < 0)
		return csntc(nick, GetReply(nick, L_NOSUCHCMD), cmd);

	if(!is_num(parv[2]) || (level = atoi(parv[2])) > OWNERLEVEL || level < 0)
		return csntc(nick, GetReply(nick, L_VALIDLEVEL));

	if(level == cs_cmds[i].level && !adm)
		return csntc(nick, "%s est dj au niveau %d.", cmd, level);

	if(!ChanCmd(i))
	{
		if(level > 4) return csntc(nick, "Le niveau d'une commande user ne doit pas dpasser 4.");
		else if(level > nick->user->level)
			return csntc(nick, GetReply(nick, L_MAXLEVELISYOURS));
		else
		{ /* dtermination du niveau admin ou pas car level 0-4 */
			if(level > 1) cs_cmds[i].flag |= CMD_ADMIN;
			else cs_cmds[i].flag &= ~CMD_ADMIN;
		}
	}
	if(ChanCmd(i) && adm) /* argument de switch admin pour les cmds chans (0-500 <= donc ncessaire)*/
	{
		if(AdmCmd(i)) cs_cmds[i].flag &= ~CMD_ADMIN;
		else cs_cmds[i].flag |= CMD_ADMIN;
	}

	cs_cmds[i].level = level;

	csntc(nick, "Le niveau de la commande %s est maintenant\2 %d\2 (%s).",
				cs_cmds[i].cmd, cs_cmds[i].level, AdmCmd(i) ? "Admin" : "User");
	write_cmds();
	BuildCommandsTable();
	return 1;
}

int disable_cmd(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	const char *cmd = parv[1];
	int disable = 0, i = GetCurCmdIndexbyName(cmd);

	if(i < 0) return csntc(nick, GetReply(nick, L_NOSUCHCMD), cmd);

	if(!strcasecmp(parv[2], "ON")) disable = 1;
	else if(!strcasecmp(parv[2], "OFF")) disable = 0;
	else return csntc(nick, GetReply(nick, L_SHOULDBEONOFF));

	if(!strcasecmp(cs_cmds[i].realcmd, "DISABLE")
		|| !strcasecmp(cs_cmds[i].realcmd, "CHCOMNAME")
		|| !strcasecmp(cs_cmds[i].realcmd, "CHLEVEL")
		|| !strcasecmp(cs_cmds[i].realcmd, "LOGIN")) /* a nuirait  tout.. */
			return csntc(nick, "Veuillez ne pas desactiver \2%s\2.", cmd);

	if(disable) cs_cmds[i].flag |= CMD_DISABLE;
	else cs_cmds[i].flag &= ~CMD_DISABLE;

	csntc(nick, "La commande %s est maintenant \2%sactive\2", cs_cmds[i].cmd, disable ? "des" : "");
	BuildCommandsTable();
	write_cmds();
	return 1;
}

#ifdef USE_WELCOMESERV
int set_motds(aNick *nick, aChan *c, int parc, char **parv)
{
	if(!strcasecmp(parv[1], "LIST"))
	{
		csntc(nick, "Motd User : \2%s\2", user_motd);
		csntc(nick, "Motd Admin: \2%s\2", admin_motd);
		return 1;
	}
	else if(parc < 2) return syntax_cmd(nick, GetCmdIndexbyName("adminmotd"));
	else if(!strcasecmp(parv[1], "USER"))
	{
		if(!strcasecmp("none", parv[2])) user_motd[0] = 0;
		else parv2msgn(parc, parv, 2, user_motd, 390);
	}
	else if(!strcasecmp(parv[1], "ADMIN"))
	{
		if(!strcasecmp("none", parv[2])) admin_motd[0] = 0;
		else parv2msgn(parc, parv, 2, admin_motd, 390);
	}

	else return csntc(nick, "Il n'y a pas de motd pour %s", parv[1]);

	if(strcasecmp("none", parv[2])) csntc(nick, "Le motd pour les %ss est maintenant:\2 %s",
							parv[1], !strcasecmp(parv[1], "USER") ? user_motd : admin_motd);
	else csntc(nick, "Le motd pour les %ss a bien t supprim", parv[1]);
	write_welcome();
	return 1;
}
#endif

/* cmds (c) keeo (le 3 en 1 par progs)*/
int globals_cmds(aNick *nick, aChan *chan, int parc, char **parv)
{
	const char *cmd = parv[1];
	char *msg = NULL;
	int i = 0;

	if(strcasecmp(cmd, "MEMO")) msg = parv2msg(parc, parv, 2, 400);

	if(msg && *msg == ':') msg++;

	if(!strcasecmp(cmd, "WALL"))
	{
		aNick *n;
		for(;i < NICKHASHSIZE;i++) for(n = nick_tab[i];n;n = n->next) csntc(n, "[\2Notice Globale\2] %s - \002%s\2", msg, nick->nick);
	}
	else if(!strcasecmp(cmd, "MSG"))
	{
		for(;i < CHANHASHSIZE;i++) for(chan = chan_tab[i];chan;chan = chan->next)
			if(chan->users > 1)	putserv("%s " TOKEN_PRIVMSG " %s :\2%s >\2 %s", cs.num, chan->chan, nick->nick, msg);
	}
	else if(!strcasecmp(cmd, "CNOTICE"))
	{
		for(;i < CHANHASHSIZE;i++) for(chan = chan_tab[i];chan;chan = chan->next)
			if(chan->users > 1) putserv("%s " TOKEN_NOTICE " %s :%s -\2 %s\2", cs.num, chan->chan, msg, nick->nick);
	}
#ifdef USE_MEMOSERV
	else if(!strcasecmp(cmd, "MEMO"))
	{
		char memo[MEMOLEN + 1];
		anUser *user;
		aNick *who;
		int min_level = 0, max_level = 0, mask = 0, owner;

		for(i = 2;i;i++)
		{
			if(*parv[i] == '-')	i++;
			else
			{
				parv2msgn(parc, parv, i, memo, MEMOLEN);
				break;
			}
		}
		min_level = getoption("-min", parv, parc, 2, 1);
		max_level = getoption("-max", parv, parc, 2, 1);
		owner = getoption("-owner", parv, parc, 2, -1);

		if(max_level > 4 || min_level > 4)
			return csntc(nick, GetReply(nick, L_VALIDLEVEL));
		mask = getoption("-user", parv, parc, 2, 0);

		for(i = 0;i < USERHASHSIZE;i++) for(user=user_tab[i];user;user = user->next)
			if(user != nick->user && (!min_level || user->level >= min_level)
			   && (!max_level || user->level <= max_level) && (!mask || !match(parv[mask], user->nick))
			   && (!owner || IsAnOwner(user)))
			{
				who = user->n;
				if(who && !(who->flag & N_AWAY))
					csntc(who, "\2MEMO DE %s :\2 %s", nick->nick, memo);
				add_memo(user, nick->user->nick, time(NULL), memo, !!(who && !(who->flag & N_AWAY)));
				write_anuser(user);
			}

		csntc(nick, "Un memo a bien t envoy  tout le monde respectant les critres donns (si spcifis) avec le texte suivant :");
		csntc(nick, "  \002%s\2", memo);
	}
#endif
	else
	{
		csntc(nick, "La sous commande %s n'existe pas", cmd);
		syntax_cmd(nick, GetCmdIndexbyName("GLOBAL"));
	}
	return 1;
}

int badmanage(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	const char *type = parv[1], *cmd = parv[2];

	if(strcasecmp(cmd, "del") && strcasecmp(cmd, "add") && strcasecmp(cmd, "list") && strcasecmp(cmd, "noreg"))
	{
		csntc(nick ,"Invalide commande: veuillez choisir 'del', 'add', 'noreg' ou 'list'");
		return syntax_cmd(nick, GetCmdIndexbyName("abad"));
	}
	if(!strcasecmp(type, "chan"))
	{
		if(!strcasecmp(cmd, "add"))
		{
			char chaine[100] = {0}, raison[150] = {0}, tmp[255];
			struct badchans *badchan;

			if(parc < 4) return csntc(nick, "Syntaxe: %s CHAN ADD <mask/chan> <raison>", RealCmd("abad"));

			if((badchan = getbadchan(parv[3])))
				return csntc(nick, "%s est dj couvert par\2 %s\2 (\002%s\2)\2.", parv[3], badchan->chaine, badchan->raison);

			Strncpy(chaine, parv[3], sizeof chaine -1);
			parv2msgn(parc, parv, 4, raison, sizeof raison - 1);/*pas de boucles inutiles avt les check*/

			badchan = add_abadchan(chaine, raison);
			csntc(nick, "La chaine\2%s\2 a bien t ajoute avec la raison:\2 %s", chaine, raison);
			csntc(nick, "Aucun salon contenant cette chaine ne pourra tre enregistr par les Users.");

			snprintf(tmp, sizeof tmp, "%s :%s", badchan->chaine, badchan->raison);
			append_file(BADCHANS_FILE, tmp);
		}
		else if(!strcasecmp(cmd, "del"))
		{
			if(parc < 3) return csntc(nick, "Syntaxe: %s CHAN DEL <mask/chan> <raison>", RealCmd("abad"));

			if(!del_abadchan(parv[3])) return csntc(nick, "La chaine %s n'existe pas", parv[3]);
			csntc(nick, "La chaine\2 %s\2 a bien t supprime", parv[3]);
			write_badchans();
		}
		else if(!strcasecmp(cmd, "list")) list_badchans(nick, chaninfo, parc, parv);
	}
	else if(!strcasecmp(type, "nick"))
	{
		if(!strcasecmp(cmd, "add"))
		{
			char chaine[NICKLEN + 1];
			struct badnicks *badnick;
			int badn, i = 0;

			if(parc < 3) return csntc(nick, "Syntaxe: %s USER ADD <mask/nick>", RealCmd("abad"));

			badn = getoption("-badnick", parv, parc, 3, -1);

			if((badnick = getbadnick(parv[3])))
				return csntc(nick, "Une chaine a dj t enregistre contenant celle ci:\2 %s\2.", badnick->chaine);

			Strncpy(chaine, parv[3], NICKLEN);
			csntc(nick, "\002%s\2 est maintenant un %s illgal", chaine, badn ? "pseudo" : "username");
			add_abadnick(chaine, badn ? 0 : CF_NOREG);
			if(!badn) csntc(nick, "L'username %s n'est dsormais plus enregistrable", chaine);
			else csntc(nick, "Tous les users qui voudront prendre ce pseudo seront deconnects au bout de %d secondes.", kill_interval);

			if((i = CountUsers(chaine))) csntc(nick, "Actuellement %d UserNames sont couverts par '%s'", i, chaine);

			append_file(BADNICKS_FILE, chaine);
		}
		else if(!strcasecmp(cmd, "del"))
		{
			if(parc < 3) return csntc(nick, "Syntaxe: %s USER ADD DEL <mask/user>", RealCmd("abad"));

			if(!del_abadnick(parv[3])) return csntc(nick, "La chaine %s n'existe pas", parv[3]);
			csntc(nick, "La chaine\2 %s\2 a bien t supprime", parv[3]);
			write_badnicks();
		}
		else if(!strcasecmp(cmd,"noreg"))
		{
			struct badnicks *badnick;
			if(parc < 3) return csntc(nick, "Syntaxe: ABAD USER NOREG <mask/nick>", RealCmd("abad"));

			if(!(badnick = getbadnick(parv[3])))
				return csntc(nick, "%s n'est pas un badnick enregistr.", parv[3]);

			badnick->flag ^= CF_NOREG;
			csntc(nick, "%s est maintenant un %s interdit", badnick->chaine,
				(badnick->flag & CF_NOREG) ? "username" : "pseudo");
		}
		else if(!strcasecmp(cmd,"list")) list_badnicks(nick, chaninfo, parc, parv);
	}
	else syntax_cmd(nick, GetCmdIndexbyName("abad"));
	return 0;
}

int rehash_conf(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	if(getoption("-help", parv, parc, 1, -1))
	{
		int i = 0;
		FILE *f;
		char buf[250], dir[50] = "syntax/";

		for (;i < cs_cmds_size; i++)
		{
			while(cs_cmds[i].cmd[0] == '\001') i++;
			strcpy(dir + 7, cs_cmds[i].cmd);
			strupr(dir + 7);
			if((f = fopen(dir, "r")))
			{
				fgets(buf, sizeof buf, f);
				strcpy(cs_cmds[i].syntax, buf);
				fclose(f);
			}
		}
		return csntc(nick, "Syntaxe recharge depuis le HDD.");
	}
	if(load_config(FICHIER_CONF) == -1)
		return csntc(nick, "Une erreur s'est produite lors de l'actualisation...");
	return csntc(nick, "Configuration actualise avec succs");
}

int showconfig(aNick *nick, aChan *c, int parc, char **parv)
{
#ifdef TESTBOT
	w2c_hallow *halw;
#endif
	csntc(nick, "Etat des variables de configuration");
	csntc(nick, "Nom du rseau : %s", network);
	csntc(nick, "Nom du serveur des Services : %s (%s) [Num: %s]", bot.server, bot.name, bot.servnum);
	csntc(nick, "IP du link : %s Port %d:", bot.ip, bot.port);
	csntc(nick, "Salon de controle : %s Salon d'aide : %s", bot.pchan, bot.chan);
	csntc(nick, "Message de quit : %s", quit_msg);
	csntc(nick, "Message PASDEPERM : %s", pasdeperm);
	csntc(nick, "Temps avant un kill : %s", duration(kill_interval));
	csntc(nick, "Change nick : %s", GetConf(CF_CHANGENICK) ? "Oui" : "Non");
	csntc(nick, "Max LastSeen : %s", duration(maxlastseen));
	csntc(nick, "Admin exempt de flood : %s", GetConf(CF_ADMINEXEMPT) ? "Oui" : "Non");
	csntc(nick, "Kill pour flood : %s", GetConf(CF_KILLFORFLOOD) ? "Oui" : "Non");
	csntc(nick, "Temps d'ignore : %s", duration(ignoretime));
	csntc(nick, "Programme de mail : %s", mailprog);
	csntc(nick, "Host hidding activ : %s Host cach : %s", GetConf(CF_HOSTHIDING) ? "Oui" : "Non", HIDDEN_HOST);
	csntc(nick, "XMODE : %s HIDECODER : %s", GetConf(CF_XMODE) ? "Oui" : "Non", GetConf(CF_HIDE_ADML4) ? "Oui" : "Non");
#ifdef TESTBOT
	csntc(nick, "Web2Cs/Port : %d", w2c_port);
	for(halw = w2c_hallowhead;halw; halw = halw->next)
		csntc(nick, "Web2Cs/HostAllow : %s", halw->host);
#endif
	return 1;
}



