/* src/ban.c - Commandes en rapport avec le ban
 * Copyright (C) 2002-2003 Inter System (Progs@Inter-System.Net)
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: ban.c,v 1.54 2003/12/29 15:47:50 romexzf Exp $
 */

#include "main.h"
#include "ban.h"
#include "outils.h"
#include "add_info.h"
#include "del_info.h"
#include "fichiers.h"
#include "cs_cmds.h"
#include "config.h"
#include <ctype.h>

/* sous traitement des commandes ban,
 * extrait/formate les arguments du ban (+erreurs) - Cesar*/
static int do_ban(aNick *nick, aChan *chan, int parc, char **parv,
	char **mask, char *raison, time_t *timeout, int *level, char **duree)
{
	int i = 2, l;

	if(parc >= 3 && is_num(parv[2])) *level = atoi(parv[i++]);
	*mask = parv[i++];
	if(parc >= i) /* raison ou dure*/
	{
		if(*parv[i] == '%') /* dure*/ *duree = ++parv[i++];
		if(parc >= i) parv2msgn(parc, parv, i, raison, 300);
	}

	l = ChanLevelbyUserI(nick->user, chan);
	if(*level < 1 || (l < *level && !IsAnAdmin(nick->user)))
		return csntc(nick, GetReply(nick, L_MAXLEVELISYOURS));

	if(*duree && (*timeout = convert_duration(*duree)) <= 0 && (**duree != '0' || !chan->bantime))
		return csntc(nick, GetReply(nick, L_INCORRECTDURATION));

	if(CNoBans(chan))
		return csntc(nick, GetReply(nick, L_NOBANSON), chan->chan);

	if(chan->banlevel > l && !IsAnAdmin(nick->user))
		return csntc(nick, GetReply(nick, L_BANLVLON), chan->banlevel, chan->chan);

	return 1;
}

static char *pretty_mask(const char *mask)
{
	static char ban[NUHLEN];
	char spec[20] = SPECIAL_CHAR "?", del = '!';
	unsigned int i = 0, j = 0, max = NICKLEN;

	for(;*mask;mask++)
	{
		if(i - j > max && *mask != del) /* partie trop longue */
		{
			if(ban[i-2] != '*') ban[i-1] = '*'; /* on tronque avec une * */
			else i--;
			while(*mask && *mask != del) mask++;/* on passe le reste jusqu'au prochain delim*/
			if(!*mask) break;
		}
		else if(isalnum(*mask) || strchr(spec, *mask) || (*mask == '*' && (!i || ban[i-1] != '*')))
			ban[i++] = *mask;/*copie des char autoriss du mask */
		else if(*mask != '*' && *mask != del) ban[i++] = '?'; /*chars inconnus remplacs par ?*/

		if(*mask == del) /*changement des dlimiteurs de la routine*/
		{
			j = i;
			ban[i++] = *mask;
			if(*mask == '!')
			{
				max = USERLEN;
				del = '@';
				strcpy(spec, SPECIAL_CHAR "?.~");
			}
			else if(*mask == '@')
			{
				max = HOSTLEN;
				del = '\0';
				strcpy(spec, "-?.");
			}
		}
	}
	if(del == '!') /* fin anormale*/ strcpy(&ban[i], "!*@*");
	else if(del == '@') strcpy(&ban[i], "@*");
	else ban[i] = 0;
	return ban;
}

/*
 * ban_cmd - Progs 11 september 2002 (c l'anniversaire des attentats qui m'y a f pens)
 * Syntaxe: BAN <chan> [level] <mask|%nick> [%dure] [raison]
 */
int ban_cmd(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	aBan *ban;
	aLink *join;
	char raison[300] = {0}, *duree = NULL, *mask = NULL;
	int level = 1;
	time_t timeout = 0;

	if(!do_ban(nick, chaninfo, parc, parv, &mask, raison, &timeout, &level, &duree))
		return 0;

	if(*mask == '%') /* old nickban*/
	{
		aNick *cible = GetMemberIbyNick(chaninfo, ++mask);
		if(!cible) return csntc(nick, GetReply(nick, L_NOTONCHAN), mask, parv[1]);
		if(!check_protect(nick, cible, chaninfo)) return 0;
		mask = getbanmask(cible, chaninfo->bantype);
	}
	else
	{
		mask = pretty_mask(mask);
		for(ban = chaninfo->banhead; ban && match(ban->mask, mask);ban = ban->next);
		if(ban)	/*recherche d'un mask couvrant le nouveau ban*/
			return csntc(nick, GetReply(nick, L_ALREADYMATCHED), mask, ban->mask, ban->level);

		for(join = chaninfo->members;join;join = join->next)
		{
			aNick *cible = join->value.j->nick;
			if(!match(mask, GetNUHbyNick(cible, 1)) && !check_protect(nick, cible, chaninfo)) return 0;
		}
	}

	csmode(chaninfo, MODE_OBV, "+b %s", mask);
	add_ban(chaninfo, mask,
			(raison[0] != '\0') ? raison : DEFRAISON,
			nick->user->nick, time(NULL),
			chaninfo->bantime ? ((timeout || (duree && *duree == '0')) ? timeout : chaninfo->bantime) : timeout,
			level);

	write_bans(chaninfo);
	return 1;
}


int kickban(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	aNick *cible;
	char raison[300] = {0}, *mask = NULL, *duree = NULL;
	int level = 1;
	time_t timeout = 0;

	if(!do_ban(nick, chaninfo, parc, parv, &mask, raison, &timeout, &level, &duree))
		return 0;

	if(!strcasecmp(cs.nick, mask)) return csntc(nick, "Non!");

	if(!(cible = GetMemberIbyNick(chaninfo, mask)))
		return csntc(nick, GetReply(nick, L_NOTONCHAN), parv[2], chaninfo->chan);

	if(!check_protect(nick, cible, chaninfo)) return 0;

	mask = getbanmask(cible, chaninfo->bantype);

	csmode(chaninfo, MODE_OBV, "+b %s", mask);
	cskick(chaninfo->chan, cible->numeric, "(\2%s\2) %s", nick->user->nick, (raison[0] != '\0') ? raison : DEFRAISON);
	add_ban(chaninfo, mask,
			(raison[0] != '\0') ? raison : DEFRAISON,
			nick->user->nick, time(NULL),
			chaninfo->bantime ? ((timeout || (duree && *duree == '0')) ? timeout : chaninfo->bantime) : timeout,
			level);
	write_bans(chaninfo);
	return 1;
}

/*
 * banlist #chan
 */
int banlist(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	aBan *ban, *tmp;
	int i = 0, m = 0, suppr = 0;
	char buf[200];

	if(parc < 2) m = 1;

	for(ban = chaninfo->banhead;ban;ban=tmp)
	{
		tmp = ban->next; /* si mis aprs, double saut (next->next) donc dpassement de la fin.*/
		if(ban->fin && (ban->fin + ban->debut) < time(NULL))
		{
			csmode(chaninfo, MODE_OBV, "-b %s", ban->mask);
			del_ban(chaninfo, ban);
			suppr = 1;
			continue;
		}
		if(m || !match(parv[2], ban->mask))
		{
			i++;
			csntc(nick, GetReply(nick, L_BLMASK), ban->mask, ban->level);
			csntc(nick, GetReply(nick, L_BLFROM), ban->de, ban->raison);

			Strncpy(buf, get_time(ban->debut), sizeof buf - 1);
			csntc(nick, GetReply(nick, L_BLEXPIRE), buf, ban->fin ? get_time(ban->debut + ban->fin) : "Jamais");
			csntc(nick, "-");
		}
	}

	if(i) csntc(nick, GetReply(nick, L_TOTALBAN), i, PLUR(i), chaninfo->chan);
	else csntc(nick, GetReply(nick, L_NOMATCHINGBAN), chaninfo->chan);
	if(suppr) write_bans(chaninfo);

	return 1;
}

/*
 * clear_bans #chan -from -minlevel -maxlevel
 */
int clear_bans(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	aBan *ban = chaninfo->banhead, *tmp = NULL;
	char modes[7] = {0}, masks[451] = {0};
	size_t size = 0, temp;
	int n = ChanLevelbyUserI(nick->user, chaninfo);
	int minlevel = getoption("-minlevel", parv, parc, 1, 1);
	int maxlevel = getoption("-maxlevel", parv, parc, 1, 1);
	int from = getoption("-from", parv, parc, 1, 0), i = 0, nb = 0, s;

	for(;ban;ban = tmp)
	{
		tmp = ban->next;
		if((from && strcasecmp(parv[from], ban->de)) || (minlevel && ban->level < minlevel)
			|| (maxlevel && ban->level > maxlevel)) continue;

		if(!IsAnAdmin(nick->user) && n < ban->level)
		{
			csntc(nick, GetReply(nick, L_CANTREMOVEBAN), ban->mask, ban->level);
			continue;
		}/* format la chaine de modes pour l'unban 6 -b max dans 450 bytes max*/

		temp = size;/* taille actuelle*/
		size += (s = strlen(ban->mask)); /* future taille*/
		if(size > 450 || nb > 5) /* buf plein ou 6 modes queued*/
		{
			modes[nb] = 0;
			csmode(chaninfo, MODE_OBV, "-%s %s", modes, masks);
			size = s, temp = 0,	nb = 0;
		}
		strcpy(&masks[temp], ban->mask);
		modes[nb++] = 'b';
		masks[size++] = ' ';
		masks[size] = 0;
		del_ban(chaninfo, ban);
		i++;
	}
	modes[nb] = 0;
	if(nb) csmode(chaninfo, MODE_OBV, "-%s %s", modes, masks);

	if(i)
	{
		write_bans(chaninfo);
		csntc(nick, GetReply(nick, L_TOTALBANDEL), i, PLUR(i), chaninfo->chan);
	}
	else csntc(nick, GetReply(nick, L_NOMATCHINGBAN), chaninfo->chan);
	return 1;
}

int unban(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	aBan *ban = chaninfo->banhead, *bant = NULL;
	const char *b = parv[2];
	int trouve = 0, n = ChanLevelbyUserI(nick->user, chaninfo);

	for(;ban;ban = bant)
	{
		bant = ban->next;
		if(!match(b, ban->mask))
		{
			if(!IsAnAdmin(nick->user) && n < ban->level)
			{
				csntc(nick, GetReply(nick, L_CANTREMOVEBAN), ban->mask, ban->level);
				continue;
			}
			csntc(nick, GetReply(nick, L_BANDELETED), ban->mask, chaninfo->chan);
			csmode(chaninfo, MODE_OBV, "-b %s", ban->mask); /* quel oubli monstrueux :)*/
			del_ban(chaninfo, ban);
			trouve = 1; /* on return pas, a permet ainsi d'unban plusieurs bans  la fois*/
		}
	}

	if(!trouve) csntc(nick, GetReply(nick, L_NOMATCHINGBAN), chaninfo->chan);
	else write_bans(chaninfo);
	return 1;
}

int unbanme(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	char *host, hostd[NUHLEN];
	aBan *bans;
	int i = 0, stillban = 0, n = ChanLevelbyUserI(nick->user, chaninfo);
#ifdef HAVE_CRYPTHOST
	char hostc[NUHLEN];
	strcpy(hostc, GetNUHbyNick(nick, 1)); /* crypt*/
#endif

	host = GetNUHbyNick(nick, 0); /* real host*/

	if(nick->user && GetConf(CF_HOSTHIDING))/* +x host*/
		snprintf(hostd, sizeof hostd, "%s!%s@%s.%s", nick->nick, nick->ident, nick->user->nick, HIDDEN_HOST);

	for(bans = chaninfo->banhead;bans;bans = bans->next)
	{
		if(!match(bans->mask, host) || (nick->user && GetConf(CF_HOSTHIDING) && !match(bans->mask, hostd))
#ifdef HAVE_CRYPTHOST
			|| !match(bans->mask, hostc)
#endif
		)
		{
			if(!IsAnAdmin(nick->user) && n < bans->level)
			{
				csntc(nick, GetReply(nick, L_CANTREMOVEBAN), bans->mask, bans->level);
				stillban = 1;
			}
			else
			{
				i++;
				csmode(chaninfo, MODE_OBV, "-b %s", bans->mask);
				csntc(nick, GetReply(nick, L_BANDELETED), bans->mask, chaninfo->chan);
				del_ban(chaninfo, bans);
			}
		}/*if match*/
	}/*for bans*/

	if(i) write_bans(chaninfo);
	csntc(nick, GetReply(nick, L_TOTALBANDEL), i, PLUR(i), chaninfo->chan);

	/* invite aprs l'unban (si complet) pour passer les +b locaux */
	if(!stillban && !getjoininfo(nick, chaninfo->chan))
		putserv("%s " TOKEN_INVITE " %s :%s", cs.num, nick->nick, chaninfo->chan);

	return 1;
}

char *getbanmask(aNick *nick, int bantype)
{
	static char mask[(HOSTLEN + USERLEN + 5)];
	char m[HOSTLEN + 1], s[10] = {0}, *host = NULL;
	unsigned int a = 0;

	if(nick->user && (nick->flag & N_CRYPT) && (host = malloc(NICKLEN + HOSTLEN + 3)))
		sprintf(host, "%s.%s", nick->user->nick, HIDDEN_HOST);
	else
#ifdef HAVE_CRYPTHOST
			host = nick->crypt;	/* choix de l'host sur laquelle*/
#else							/* le ban va porter: relle, crypte, +x*/
			host = nick->host;	/* selon les 'protections' actuelles de *nick*/
#endif

	if(bantype == 1 || !bantype) /*ban du type *!*ident@*.host */
	{
		if((sscanf(host, "%u.%9s", &a, s) == 2 && strlen(s) == 8) || is_ip(host))
        	sprintf(m, "%d.*", a);
        else snprintf(m, HOSTLEN, "*%s", strchr(host, '.'));
        sprintf(mask, "*!*%s@%s", nick->ident, m);
    }
    else if(bantype == 2) sprintf(mask, "*!*%s@*", nick->ident); /*type : *!*ident@* */
	else if(bantype == 3) sprintf(mask, "*%s*!*@*", nick->nick); /*type : *nick*!*@* */
	else if(bantype == 4) sprintf(mask, "*!*@%s", host); /*type : *!*@host */
	else sprintf(mask, "*!*%s@%s", nick->ident, host); /*type *!*ident@host*/

	if(nick->user && nick->flag & N_CRYPT) free(host);
	return mask;
}

aBan *is_ban(aNick *nick, aChan *chan) /* retourne un pointeur vers le ban*/
{									/* vrifie sur toutes les hosts (sauf IP)*/
	aBan *b = chan->banhead;
	char *host, hostd[NUHLEN];

#ifdef HAVE_CRYPTHOST
	char hostc[NUHLEN];
	strcpy(hostc, GetNUHbyNick(nick, 1));
#endif

	host = GetNUHbyNick(nick, 0); /* real host*/

	if(nick->user && GetConf(CF_HOSTHIDING))
		snprintf(hostd, sizeof hostd, "%s!%s@%s.%s", nick->nick, nick->ident, nick->user->nick, HIDDEN_HOST);

	for(;b;b = b->next)
		if(!match(b->mask, host) || (nick->user && GetConf(CF_HOSTHIDING) && !match(b->mask, hostd))
#ifdef HAVE_CRYPTHOST
			|| !match(b->mask, hostc)
#endif
		) return b;

	return NULL;
}

char *GetBanType(aChan *chan)
{
	static char mask[20];
	switch(chan->bantype)
	{
		case 1: strcpy(mask, "*!*ident@*.host"); break;
		case 2: strcpy(mask, "*!*ident@*"); break;
		case 3: strcpy(mask, "*nick*!*@*"); break;
		case 4: strcpy(mask, "*!*@host"); break;
		case 5: strcpy(mask, "*!*ident@host"); break;
		default: strcpy(mask, "<invalide>");
	}
	return mask;
}
