/* src/outils.c - Divers outils
 * Copyright (C) 2002-2003 Inter System
 *
 * contact: Progs_and_davidounet@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: outils.c,v 1.129 2003/12/27 16:19:27 romexzf Exp $
 */

#include <ctype.h>
#include <stdarg.h>
#include "main.h"
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "debug.h"
#include "cs_cmds.h"
#include "hash.h"
#include "outils.h"
#ifdef HAVE_CRYPTHOST
#include <crypt.h>
#endif

static const char *months[12] = {
  "Janvier", "Fevrier", "Mars", "Avril", "Mai", "Juin", "Juillet", "Aot", "Septembre", "Octobre", "Novembre", "Decembre"
};

static const char *weekdays[7] = {
  "Dimanche", "Lundi", "Mardi", "Mercredi", "Jeudi", "Vendredi", "Samedi"
};

static const struct Modes {
	char mode;
	int value;
} UMode[] = {
	{ 'i', N_INV },
	{ 'x', N_CRYPT },
	{ 'r', N_REG },
	{ 'o', N_OPER },
	{ 'k', N_SERVICE },
	{ 'a', N_ADM },
	{ 'p', N_GOD },
	{ 'f', N_FEMME },
	{ 'h', N_HOMME },
	{ 'g', N_DEBUG },
	{ 'w', N_WALLOPS },
	{ 'D', N_DIE },
	{ 'd', N_DEAF },
	{ 'H', N_SPOOF },
},
	CMode[] = {
	{ 'n', C_MMSG },
	{ 't', C_MTOPIC },
	{ 'i', C_MINV },
	{ 'l', C_MLIMIT },
	{ 'k', C_MKEY },
	{ 's', C_MSECRET },
	{ 'p', C_MPRIVATE },
	{ 'm', C_MMODERATE },
	{ 'c', C_MNOCTRL },
	{ 'C', C_MNOCTCP },
	{ 'O', C_MOPERONLY },
	{ 'r', C_MUSERONLY },
};

#ifdef HAVE_CRYPTHOST
char *hostprot(const char *host)
{
	u_int32_t sum, digest[4];
	static char newhost[HOSTLEN + 1];
	int ip = is_ip(host);
	char key1[HOSTLEN + 1], *key2 = strchr(host, '.'), *key = ip ? key2 : key1;

	Strncpy(key1, host, HOSTLEN);
	strtok(key1, ".");

	if(!host[0] % 2)
	{
		MD2_CTX context;
		MD2Init(&context);
		MD2Update(&context, (unsigned char *) key, strlen(key));
		MD2Update(&context, (unsigned char *) host, strlen(host));
		MD2Final((void *) digest, &context);
	}
	else
	{
		MD5_CTX context;
		MD5Init(&context);
		MD5Update(&context, (unsigned char *) key, strlen(key));
		MD5Update(&context, (unsigned char *) host, strlen(host));
		MD5Final((void *) digest, &context);
	}

	sum = digest[0] + digest[1] + digest[2] + digest[3];

	if (!ip) snprintf(newhost, sizeof newhost, "%X%s", sum, key2);
	else snprintf(newhost, sizeof newhost, "%s.%X", key1, sum);
	return newhost;
}
#endif

int chan_check(const char *chan, aNick *nick)
{
	struct badchans *badchan;
	const char *p = chan;

	if(strlen(chan) > REGCHANLEN)
		return csntc(nick, GetReply(nick, L_CHANMAXLEN), REGCHANLEN);

	if(*p != '#' || strchr(chan, ','))
		return csntc(nick, GetReply(nick, L_USER_INVALID), chan);

	while(*p)
		if(!isascii(*p++))
			return csntc(nick, GetReply(nick, L_USER_INVALID), chan);

	if((badchan = getbadchan(chan)))
		return csntc(nick, GetReply(nick, L_ILLEGALCHAN), chan, badchan->raison);

	return 1;
}

int is_num(const char *num)
{
   while(*num) if(!isdigit(*num++)) return 0;
   return 1;
}

int is_ip(const char *ip)
{
	char n = 0;
	unsigned int a = 0, b = 0, c = 0, d = 0;
	if(4 == sscanf(ip, "%u.%u.%u.%u%c\n", &a, &b, &c, &d, &n)
		&& a < 256 && b < 256 && c < 256 && d < 256 && n == 0)
			return 1;
	return 0;
}

void strupr(char *buf)
{
	for(;*buf;buf++) *buf = toupper(*buf);
}

void strlwr(char *buf)
{
	for(;*buf;buf++) *buf = tolower(*buf);
}

void conv_char(const char *chan, char *string)
{
	int index = 0;
	*string = '\0';

	while(*chan)
	{
		if(!isalnum(*chan))
		{
			sprintf(string + index, "%%%03o", *chan);
			index += 4;
		}
		else string[index++] = tolower(*chan);
		chan++;
	}
	string[index] = '\0';
	return;
}

void strip_newline(char *string)
{
   register char *p = string;
   while(*p && *p != '\n' && *p != '\r') p++;
   *p = '\0';
}

anUser *GetUserIbyMail(const char *mail)
{
	anUser *u;
	int i = 0;
	for(;i < USERHASHSIZE;i++) for(u = user_tab[i];u;u = u->next) if(!strcasecmp(mail, u->mail)) return u;
	return NULL;
}

int ChanLevelbyUserI(anUser *user, aChan *chan)
{
	anAccess *acces = GetAccessIbyUserI(user, chan);
	return acces ? acces->level : 0;
}

aJoin *getjoininfo(aNick *nick, const char *chan)
{
	register aJoin *join = nick->joinhead;
	for(;join && strcasecmp(chan, join->channel);join = join->next);
	return join;
}

struct badnicks *getbadnick(const char *chan)
{
   struct badnicks *tmp = badnhead;
   for(;tmp && match(tmp->chaine, chan);tmp = tmp->next);
   return tmp;
}

int IsBadNick(const char *chan)
{
   struct badnicks *tmp;
   for(tmp = badnhead;tmp;tmp = tmp->next) if(!match(tmp->chaine, chan)) return 1;
   return 0;
}

struct badchans *getbadchan(const char *chan)
{
   struct badchans *current = badchead;
   for(;current && match(current->chaine, chan);current = current->next);
   return current;
}

char *IsAnOwner(anUser *user)
{
	anAccess *acces = user->accesshead;
	for(;acces && acces->level != OWNERLEVEL;acces = acces->next);
	return acces ? acces->c->chan : NULL;
}

anAccess *GetAccessIbyUserI(anUser *user, aChan *chan)
{
	anAccess *a = user->accesshead;
	for(;a && a->c != chan; a = a->next);
	return (a && !AWait(a)) ? a : NULL;
}

aNick *GetMemberIbyNum(aChan *chan, const char *num)
{
	register aLink *l = chan->members;
	for(;l && strcmp(l->value.j->nick->numeric, num);l = l->next);
	return l ? l->value.j->nick : NULL;
}

aNick *GetMemberIbyNick(aChan *chan, const char *nick)
{
	aLink *l = chan->members;
	for(;l && strcasecmp(l->value.j->nick->nick, nick);l = l->next);
	return l ? l->value.j->nick : NULL;
}

/*
 * getoption:
 * recherche dans un tableau char ** (de 'parc' entres)
 * l'expression "opt"  partir de l'lment 'deb'
 * -si exp_int est > 0:
 *  cela veut dire qu'on attend une valeur numerique apres le char 'opt'
 *  si c'est le cas, la fct renvoit la valeur, sinon 0
 * -si exp_int est < 0:
 *  cela veut dire que l'on recherche uniquement si 'opt' est dans parv,
 *  si c'est le cas, retourne 1
 * -si exp_int est = 0:
 *  cela veut dire que l'on attend une string apres le 'opt',
 *  si c'est le cas retourne l'index de cette string dans parv, sinon 0.
 *
 */

int getoption(const char *opt, char **parv, int parc, int deb, int exp_int)
{
	for(;deb <= parc;deb++)
		if(!strcasecmp(opt, parv[deb]))
		{
			if(exp_int < 0) return 1;
			if(++deb <= parc)
			{
				if(is_num(parv[deb]) && exp_int) return atoi(parv[deb]);
				else if(!exp_int) return deb;
			}
			break;
		}
	return 0;
}

aServer *num2servinfo(const char *num)
{
	unsigned int servindex = base64toint(num);

	if(servindex > MAXNUM-1) return NULL;
	else return serv_tab[servindex];
}

aServer *GetLinkIbyServ(const char *serv)
{
	int i = 0;
	for(;i < MAXNUM;i++) if(serv_tab[i] && !strcasecmp(serv_tab[i]->serv, serv)) return serv_tab[i];
	return NULL;
}

void parv2msgn(unsigned int parc, char **parv, unsigned int base, char *buf, unsigned int size)
{
	unsigned int i = base, toksize, written = 0;

	for(;i <= parc;i++)
	{
		toksize = strlen(parv[i]);

		if(i > base) buf[written++] = ' ';

		if(written + toksize >= size-1)
		{
			Strncpy(buf + written, parv[i], size - written - 1);
			written = size - 1;
			break;
		}
		else
		{
			strcpy(buf + written, parv[i]);
			written += toksize;
		}
	}
	buf[written] = 0;
	return;
}


char *parv2msg(unsigned int parc, char **parv, unsigned int base, unsigned int size)
{
	unsigned int i = base, toksize, written = 0;
	static char buf[500];

	if(size > sizeof buf) size = sizeof buf;

	for(;i <= parc;i++)
	{
		toksize = strlen(parv[i]);

		if(i > base) buf[written++] = ' ';

		if(written + toksize >= size-1)
		{
			Strncpy(buf + written, parv[i], size - written - 1);
			written = size - 1;
			break;
		}
		else
		{
			strcpy(buf + written, parv[i]);
			written += toksize;
		}
	}
	buf[written] = 0;
	return buf;
}

char *duration(int secs)
{
	int mins = 0, hours = 0, days = 0, i = 0;
	static char dur[200];
	dur[0] = '\0';

	if(secs >= 60)
	{
		mins = secs / 60;
		secs -= 60 * mins;
	}
	if(mins >= 60)
	{
		hours = mins / 60;
		mins -= 60 * hours;
	}
	if(hours >= 24)
	{
		days = hours / 24;
		hours -= 24 * days;
		i = sprintf(dur, "\002%d\2 jours ", days);
	}
	if(hours) i += sprintf(dur + i, "\002%d\2 heures ", hours);
	if(mins) i += sprintf(dur + i, "\002%d\2 minutes ", mins);
	if(secs) sprintf(dur + i,  i ? "et \002%d\2 secondes" : "\002%d\2 secondes", secs);
	else dur[i-1] = 0;
	return dur;
}

time_t convert_duration(const char *p)/* extrait une dure en secondes*/
{										/* d'un format XdXhXm.. - Cesar*/
	const char *p1 = p;
	time_t result = 0;

	while(*p1)
	{
		if(!isdigit(*p1) && *p1 != 'h' && *p1 != 'H' && *p1 != 'j'
			&& *p1 != 'J' && *p1 != 'd' && *p1 != 'D' && *p1 != 'Y'
			&& *p1 != 'y' && *p1 != 'm')
		{
			result = -1;
			break;
		}
		switch(*p1)
		{
			case 'H': case 'h':
						result += 3600 * strtol(p, NULL, 10);
						p = p1 + 1;
						break;
			case 'm':
						result += 60 * strtol(p, NULL, 10);
						p = p1 + 1;
						break;
			case 'Y': case 'y':
						result += 31536000 * strtol(p, NULL, 10);
						p = p1 + 1;
						break;
			case 'd': case 'j':
			case 'J': case 'D':
						result += 86400 * strtol(p, NULL, 10);
						p = p1 + 1;
						break;
		}
		p1++;
	}
	if(result >= 0) result += strtol(p, NULL, 10);
	return result;
}

char *get_time(time_t ttime)
{
	register struct tm *lt = localtime(&ttime);
	static char timet[70];

	snprintf(timet, sizeof timet, "%s %d %s %d %02d:%02d:%02d",
		weekdays[lt->tm_wday], lt->tm_mday, months[lt->tm_mon],
		1900 + lt->tm_year, lt->tm_hour, lt->tm_min, lt->tm_sec);

	return timet;
}

char *GetNUHbyNick(aNick *nick, int type)
{
	static char nuhbuf[NUHLEN];
	register char *ptr = nick->nick, *ptr2 = nuhbuf;

	while((*ptr2++ = *ptr++));
	*(ptr2-1) = '!';
	for(ptr = nick->ident;(*ptr2++ = *ptr++););
	*(ptr2-1) = '@';
	for(
#ifdef HAVE_CRYPTHOST
		ptr = (type == 1) ? nick->crypt : nick->host
#else
		ptr = nick->host
#endif
	;(*ptr2++ = *ptr++););

	return nuhbuf;
}

char *GetPrefix(aNick *nick)
{
	static char pref[12];
	pref[0] = '\0';

	if(IsOper(nick)) strcpy(pref, "\2\0037*\2\003");
	if(nick->user)
	{
		if(IsAnAdmin(nick->user)) strcat(pref, "\00312^\003");
		else strcat(pref, "\0035!\003");
	}
	return pref;
}

char *GetChanPrefix(aNick *nick, aJoin *join, int colors)
{
	static char pref[14];
	pref[0] = '\0';

	if(IsOper(nick)) strcpy(pref, colors ? "\2\0037*\2\003" : "*");
	if(nick->user)
	{
		if(IsAnAdmin(nick->user)) strcat(pref, colors ? "\00312^\003" : "^");
		else strcat(pref, colors ? "\0035!\003" : "!");
	}
	if(IsOp(join)) strcat(pref, "@");
	if(IsVoice(join)) strcat(pref, "+");

	return pref;
}

char *GetUserOptions(anUser *user)
{
	static char option[120];
	int i = 0;
	option[0] = '\0';

	if(UNopurge(user)) strcpy(option, "NoPurge "), i += 8;
	if(UNoMemo(user)) strcpy(option + i, "NoMemo "), i += 7;
	if(UFirst(user)) strcpy(option + i, "Registering "), i += 12;
	if(UOubli(user)) strcpy(option + i, "Oubli "), i += 6;
	if(UWantX(user)) strcpy(option + i, "VirtualHost "), i += 12;
	if(user->flag & U_ALREADYCHANGE) strcpy(option + i, "UserChanged "), i += 12;
	if(CantRegChan(user)) strcpy(option + i, "CantRegChan "), i += 12;
#ifdef USE_NICKSERV
	if(UPKill(user)) strcpy(option + i, "Protection(Kill) "), i += 17;
	else if(UPNick(user)) strcpy(option + i, "Protection(ChNick) "), i += 19;
#endif
	if(user->flag & U_PREJECT) strcpy(option + i, "Accs(Rejet)");
	else if(user->flag & U_POKACCESS) strcpy(option + i, "Accs(Ask)");
	else if(user->flag & U_PACCEPT) strcpy(option + i, "Accs(Accepte)");

	if(option[0] == '\0') strcpy(option, "Aucune");
	return option;
}

char *GetAccessOptions(anAccess *acces)
{
	static char options[100];
	int i = 0;
	options[0] = '\0';

	if(AOp(acces)) strcpy(options, " autoop"), i += 7;
	if(AProtect(acces)) strcpy(options + i, " protg"), i += 8;
	if(AVoice(acces)) strcpy(options + i, " autovoice"), i += 10;
	if(ASuspend(acces)) strcpy(options + i, " suspendu");
	if(options[0] == '\0') strcpy(options, " Aucune");
	return options;
}

char *GetChanOptions(aChan *chan)
{
	static char coptions[300];
	int i = 0;
	coptions[0] = 0;

	if(CLockTopic(chan)) strcpy(coptions, " locktopic"), i += 10;
	if(CNoOps(chan)) strcpy(coptions + i, " noops"), i += 6;
	if(CNoBans(chan)) strcpy(coptions + i, " nobans"), i += 7;
	if(CStrictOp(chan)) strcpy(coptions + i, " strictop"), i += 9;
	if(CSetWelcome(chan)) strcpy(coptions + i, " welcome"), i += 8;
	if(CAutoInvite(chan)) strcpy(coptions + i, " autoinvite"), i += 11;
	if(chan->flag & C_ALREADYRENAME) strcpy(coptions + i, " renomm"), i += 8;
	if(CFLimit(chan)) sprintf(coptions + i, " auto-limit(%d,%d)", chan->limit_inc, chan->limit_min);
	if(coptions[0] == 0) strcpy(coptions, " Aucune");

	return coptions;
}

int checkpass(const char *pass, anUser *user)
{
	return !strcmp(mkpass(pass), user->passwd);
}

void putlog(const char *fichier, const char *fmt, ...)
{
	va_list vl;
	FILE *fp;

	if(!fichier || !*fichier || !(fp = fopen(fichier, "a"))) return;

	fputc('[', fp);
	fputs(get_time(time(NULL)), fp);
	fputs("] ", fp);

	va_start(vl, fmt);
	vfprintf(fp, fmt, vl);
	va_end(vl);

	fputc('\n', fp);
	fclose(fp);
}

char *make_servnum(int c)
{
	static char res[3];
	const char base64[64] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789[]";
#ifndef N2K
	res[0] = base64[c & 63];
	res[1] = 0;
#else
	res[0] = base64[c / 64];
	res[1] = base64[c & 63];
	res[2] = '\0';
#endif

	return res;
}

unsigned long int resolve(const char *host)
{
	struct hostent *he = gethostbyname(host);

	if(!he) return inet_addr(host);
	return *(int *)he->h_addr;
}

int check_protect(aNick *executant, aNick *execute, aChan *chan)
{

   anAccess *access1, *access2;

   if(executant == execute) return 1;
   else if((execute->flag & (N_GOD|N_SERVICE)) && !IsOper(executant) && !IsAnAdmin(executant->user))
   		return csntc(executant, "L'user \2%s\2 est protg", execute->nick);
   else if(!execute->user) return 1;
   else if(IsAnAdmin(executant->user) && executant->user->level > execute->user->level) return 1;
   /*l'executant est un admin de level suprieur a l'execut*/
   else if(IsAnAdmin(executant->user) && IsAnAdmin(execute->user)/* cible admin >*/
   		&& executant->user->level <= execute->user->level && executant->user->level < 4)
      return csntc(executant, "L'user \2%s\2 est un Admin de niveau suprieur ou gal au votre.", execute->nick);

   else if(!IsAnAdmin(executant->user) && IsAnAdmin(execute->user))/*executant n'est pas admin mais execute est admin*/
   		return csntc(executant, "L'user \2%s\2 est un Administrateur des Services.", execute->nick);

    /*optimisation: on check dabord l'acces de l'execut!*/
   if(!(access2 = GetAccessIbyUserI(execute->user, chan)) || ASuspend(access2) || !AProtect(access2)) return 1;

   /*on boucle les access de l'executant pour voir si on en trouve 1 sur le chan*/
   if(!(access1 = GetAccessIbyUserI(executant->user, chan)) || ASuspend(access1)) return 0;
   /*ne doit jamais arriver, voudrait dire que executant n'est pas un admin et n'a pas d'access sur le chan*/

   if(access1->level > access2->level) return 1; /*on verifie si executant a un level superieur a execute, ou si execute n'est pas protect;*/

   csntc(executant, "L'user \2%s\2 est protg ou a un access suprieur au votre sur \2%s\2.", execute->nick, chan->chan);
   return 0; /*arrive pour tous les autres cas*/
}

int parse_umode(int flag, const char *p)
{/* flag de dpart*/
	int w = 1, i;

	if(!p) return flag;
	for(;*p;p++)
	{
		if(*p == '+') w = 1;
		else if(*p == '-') w = 0;
		else
			for (i = 0;i < ASIZE(UMode);i++)
				if(*p == UMode[i].mode)
				{
					if(w) flag |= UMode[i].value;
					else flag &= ~UMode[i].value;
					break;
				}
	}
	return flag;
}

char *GetModes(int flag)
{
	static char mode[ASIZE(UMode)];
	int i = 0, j = 0;
	for(;i < ASIZE(UMode);i++) if(flag & UMode[i].value) mode[j++] = UMode[i].mode;
	mode[j] = '\0';
	return mode;
}

int cmodetoflag(int flag, const char *mode)
{
	int w = 1, i;

	if(!mode) return flag;
	for(;*mode;mode++)
	{
		if(*mode == '+') w = 1;
		else if(*mode == '-') w = 0;
		else for (i = 0;i < ASIZE(CMode);i++)
				if(*mode == CMode[i].mode)
				{
					if(w) flag |= CMode[i].value;
					else flag &= ~CMode[i].value;
					break;
				}
	}
	return flag;
}

int string2scmode(struct cmode *mode, const char *modes, const char *key, const char *limit)
{
    int newmode = cmodetoflag(mode->modes, modes);

	if(newmode & C_MLIMIT)
	{
		int lim = 0;
		if(limit && is_num(limit) && (lim = atoi(limit))) mode->limit = lim;
		else if(!(mode->modes & C_MLIMIT)) newmode &= ~C_MLIMIT;/* +l mis mais aucune limite valide..*/
	}
	else mode->limit = 0;

	if(newmode & C_MKEY)
	{
		if(key && *key) Strncpy(mode->key, key, KEYLEN);
		else if (!*mode->key) newmode &= ~C_MKEY;/* +k mis pourtant aucun cl valide..*/
	}
	else if(mode->modes & C_MKEY && (!key || !*key)) newmode |= C_MKEY;/* -k autoris seulement si key fournie*/
	else *mode->key = 0;
	return (mode->modes = newmode);
}

char *GetCModes(struct cmode modes)
{
	static char mode[ASIZE(CMode) + KEYLEN + 20];
	int i = 0, j = 0;
	for(;i < ASIZE(CMode);i++) if(modes.modes & CMode[i].value) mode[j++] = CMode[i].mode;

	if(modes.modes & C_MLIMIT) j += sprintf(mode + j, " %d", modes.limit);
	if(modes.modes & C_MKEY)
	{
		mode[j++] = ' ';
		strcpy(mode + j, modes.key);
	}
	else mode[j] = '\0';
	return mode;
}

int IsValidNick(char *p)
{
	if(*p == '-' || isdigit(*p)) return 0;
	while(*p)
	{
		if(!isalnum(*p) && !strchr(SPECIAL_CHAR, *p)) return 0;
		p++;
	}
	return 1;
}

int IsValidMail(char *p)
{
	int fa = 0, fd = 0, ispoint = 0;
	if(strchr("-_.@", *p)) return 0;
	while(*p)
	{
		if(*p == '@')
		{
			if(!fa) fa = 1;
			else return 0;
		}
		if(fa && *p != '.') ispoint = 0;
		if(fa && *p == '.')
		{
			if(ispoint) return 0; /* le char d'avant etait un point, 2 points successifs dans l'url interdits */
			fd = 1;
			ispoint = 1;
		}
		if(!isalnum(*p) && *p != '.' && *p != '-' && *p != '_' && *p != '@') return 0;
		p++;
	}
	if(fa && fd) return 1;
	return 0;
}

int GetCmdIndexbyName(const char *name)
{
	unsigned int i = 0;

	for(;i < cs_cmds_size;i++) if(!strcasecmp(name, cs_cmds[i].realcmd)) return i;
	return -1;
}

int GetCurCmdIndexbyName(const char *name)
{
	unsigned int i = 0;

	for(;i < cs_cmds_size;i++) if(!strcasecmp(name, cs_cmds[i].cmd)) return i;
	return -1;
}

char *RealCmd(const char *cmd)
{
	static char realcmd[15];
	int i = GetCmdIndexbyName(cmd);

	realcmd[0] = '\0';
	if(i >= 0) strcpy(realcmd, cs_cmds[i].cmd);
	return realcmd;
}

anUser *ParseNickOrUser(aNick *nick, char *arg)
{
	anUser *user = NULL;

	if(*arg == '%') /* recherche sur nick actif! */
	{
		aNick *n;
		if(*++arg == 0) csntc(nick, "Veuillez spcifier un nick aprs '%%' : COMMANDE %%nick-connect [args..]");
		else if(!(n = getnickbynick(arg))) csntc(nick, GetReply(nick, L_NOSUCHNICK), arg);
		else if(!n->user) csntc(nick, GetReply(nick, L_NOTLOGUED), arg);
		else return n->user;
	}
	else if(!(user = getuserinfo(arg))) csntc(nick, GetReply(nick, L_NOSUCHUSER), arg);
	return user;
}

void File_UserDel(char *nick)
{
	char path[5 * NICKLEN] = DBDIR "/usr/";

	conv_char(nick, path + strlen(path));
	remove(path);
	return;
}

char *str_dup(char **to, const char *from)
{
	if(from && *from)
	{
		*to = realloc(*to, strlen(from) + 1);
		if(!to) Debug(W_MAX, "EXIT: strdup: no memory for 'to'(%p)=%s 'from'(%p)=%s\n",
			(void *) *to, *to, (void *) from, from);
		else strcpy(*to, from);
	}
	return *to;
}

char *Strncpy(char *to, const char *from, size_t n)/* copie from dans to. ne copie que n char*/
{							/* MAIS AJOUTE LE CHAR \0  la fin, from DOIT donc faire n+1 chars.*/
	const char *end = to + n;
	char *save = to;
	while(to < end && (*to++ = *from++));
	*to = 0;
	return save;
}

char *Strtok(char **save, char *str, char sep)/* fonction tire d'ircu, simplifie*/
{
	register char *pos = *save; 	/* keep last position across calls */
	char *tmp;

	if(str) pos = str; 	/* new string scan */

	while(pos && *pos && sep == *pos) pos++; /* skip leading separators */

	if (!pos || !*pos) return (pos = *save = NULL); /* string contains only sep's */

	tmp = pos; /* now, keep position of the token */

	if((pos = strchr(pos, sep))) *pos++ = 0;/* get sep -Cesar*/

	*save = pos;
	return tmp;
}

int switch_option(aNick *nick, const char *choix, const char *opt, const char *targ, int *flagto, int flag)
{
	if(!choix) *flagto ^= flag;
	else if(!strcasecmp(choix, "on")) *flagto |= flag;
	else if(!strcasecmp(choix, "off")) *flagto &= ~flag;
	else return csntc(nick, GetReply(nick, L_SHOULDBEONOFF));

	if(*flagto & flag) csntc(nick, GetReply(nick, L_OPTIONNOWON), opt, targ);
	else csntc(nick, GetReply(nick, L_OPTIONNOWOFF), opt, targ);
	return 0;
}

