/* src/socket.c - Gestion des sockets & parse
 * Copyright (C) 2002-2003 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: socket.c,v 1.4 2003/12/27 16:19:27 romexzf Exp $
 */

#include <unistd.h>
#include "main.h"
#include "outils.h"
#include "serveur.h"
#include "socket.h"
#include "hash.h"
#include "debug.h"
#include "cs_cmds.h"
#include "add_info.h"
#ifdef USEBSD
#include <netinet/in.h>
#endif
#include <arpa/inet.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <errno.h>
#ifdef TESTBOT
#include "web2cs.h"
#endif

static struct aMsg {
  const char *cmd;
  int (*func) (int parc, char **parv);
} msgtab[] = {
  {TOKEN_PRIVMSG, m_privmsg},
  {TOKEN_PING, m_ping},
  {TOKEN_NICK, m_nick},
  {TOKEN_JOIN, m_join},
  {TOKEN_PART, m_part},
  {TOKEN_MODE, m_mode},
  {TOKEN_KICK, m_kick},
  {TOKEN_CREATE, m_create},
  {TOKEN_QUIT, m_quit},
  {TOKEN_BURST, m_burst},
  {TOKEN_AWAY, m_away},
  {TOKEN_SERVER, m_server},
#ifdef N2K
  {"SERVER", m_server},
  {"OM", m_mode},
  {"CM", m_clearmode},
#endif
  {TOKEN_SQUIT, m_squit},
  {TOKEN_TOPIC, m_topic},
  {TOKEN_KILL, m_kill},
  {TOKEN_WHOIS, m_whois},
#ifndef N2K
  {TOKEN_SETHOST, m_sethost},
  {TOKEN_SETIDENT, m_setident},
  {TOKEN_ACCOUNT, m_account},
#endif
  {TOKEN_PASS, m_pass},
  {TOKEN_EOB, m_eob},
};

static void parse_this(char *msg)
{
	int parc = 0, i = 0;
	char *cmd = NULL;
	char *parv[MAXPARA + 2];
#ifdef DEBUG
	char logg[513];
	Strncpy(logg, msg, 512);
#endif

	if(!mainhub) parv[parc++] = bot.server;
#ifndef N2K
	if(*msg == ':') msg++;
#endif
	while(*msg)
	{
		while(*msg == ' ') *msg++ = 0;/* on supprime les espaces pour le dcoupage*/
		if(*msg == ':') /* last param*/
		{
			parv[parc++] = msg + 1;
			break;
		}
		if(!*msg) break;

		if(parc == 1 && !cmd) cmd = msg;/* premier passage  parc 1 > cmd*/
		else parv[parc++] = msg;
		while(*msg && *msg != ' ') msg++;/*on laisse passer l'arg..*/
	}
	parv[parc] = NULL;

#ifdef DEBUG /*etical loging*/
	if(parc <= 1 || ((strcasecmp(cmd, TOKEN_PRIVMSG) || *parv[1] != '#')
		&& strcasecmp(cmd, TOKEN_PING) && strcasecmp(cmd, TOKEN_PONG)))
			putlog(LOG_PARSES, "R - %s", logg);
#endif

	for(;i < ASIZE(msgtab);i++)
		if (!strcasecmp(msgtab[i].cmd, cmd))
		{
			msgtab[i].func(parc, parv);
			return;/*on sait jamais */
		}
}

int init_bot(unsigned long int ip)
{
	int sock;
	time_t LTS = time(NULL);
	struct sockaddr_in fsocket;
	char num[7];

	sock = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
	if(sock < 0) return -1;

	fsocket.sin_family = AF_INET;
	fsocket.sin_addr.s_addr = ip;
	fsocket.sin_port = htons(bot.port);

	if(connect(sock, (struct sockaddr *) &fsocket, sizeof fsocket) == -1)
	{
		Debug(W_MAX, "Erreur #%d lors de la connexion: %s", errno, strerror(errno));
		close(sock);
		return -1;
	}

	bot.sock = sock;
	mainhub = NULL;
	putserv("PASS %s", bot.pass);
#ifdef N2K/* num + max user*/
	sprintf(num, "%sAAD", bot.servnum);
#else
	sprintf(num, "%sAD", bot.servnum);
#endif

	putserv("SERVER %s 1 %lu %lu J10 %s +s :%s", bot.server, bot.uptime, LTS, num, bot.name);
	add_server(bot.server, num, "1", "J10", bot.server);

	putserv("%s " TOKEN_NICK " %s 1 %lu %s %s %s B]AAAB %s :%s", bot.servnum, cs.nick, LTS, cs.ident, cs.host, cs.mode, cs.num, cs.name);
	add_nickinfo(cs.nick, cs.ident, cs.host, "B]AAAB", cs.num, num2servinfo(bot.servnum), cs.name, LTS, cs.mode);

#ifdef USE_WELCOMESERV
	putserv("%s " TOKEN_NICK " %s 1 %lu %s %s %s B]AAAB %s :%s", bot.servnum, ws.nick, LTS, ws.ident, ws.host, ws.mode, ws.num, ws.name);
	add_nickinfo(ws.nick, ws.ident, ws.host, "B]AAAB", ws.num, num2servinfo(bot.servnum), ws.name, LTS, ws.mode);
#endif

	return sock;
}

int run_bot(void)
{
	char tbuf[513] = {0};
	int t = 0, read, sock = bot.sock;
	char bbuf[513];/* fonction commune de parsage de l'ircsock*/
	register char *ptr;
#ifndef TESTBOT
	time_t next_ping = (time(NULL) + PING_FREQ), waitping = 0;
	fd_set master;

	FD_ZERO(&master);

#else

	w2c_setnonblocking(sock);
	if(sock > highsock) highsock = sock;

#endif

	FD_SET(sock, &master);

	while(!want_to_quit)
	{

#ifdef TESTBOT
		if(w2c_checksocks())
		{
			int i = 0;

			for(;i <= highsock;i++)
			{
				if(!FD_ISSET(i, &socks)) continue;
				if(i == bot.sock)
				{
#else

		if(!FD_ISSET(sock, &master))
		{
			if(next_ping <= time(NULL))
			{
				if(!waitping)
				{
					struct timeval tv;
					gettimeofday(&tv, NULL);
					putserv("%s " TOKEN_PING " !%ld.%ld %s %ld.%ld", bot.servnum, tv.tv_sec, tv.tv_usec, mainhub->num, tv.tv_sec, tv.tv_usec);
					next_ping = tv.tv_sec + PING_FREQ;
					waitping = 1;
				}
				else
				{
					putlog(LOG_ERREURS, "Pas de rponse pour %s", bot.ip);
					close(sock);
					break;
				}
			}
		}
		else
		{
#endif /* TESTBOT*/

			/* lecture du buffer de recv(), puis dcoupage selon les \r\n et,
			enfin on garde en mmoire dans tbuf le dbut d'un paquet IRC s'il
			est incomplet. au select suivant les bytes lui appartenant lui seront
			ajoute grace  t, index permanent.       packet1\r\npacket2\r\n
			la lecture de bloc de 512 bytes plutot que byte  byte est
			au moins 3 fois plus rapide!*/
			read = recv(sock, bbuf, sizeof bbuf -1, 0);
			ptr = bbuf;

			if(read <= 0 && errno != EINTR)
			{
				close(sock);
				return Debug(W_MAX, "Erreur lors de recv(): [%s]", strerror(errno));
			}
#ifndef TESTBOT
			next_ping = time(NULL) + PING_FREQ;
			waitping = 0;
#endif
			bbuf[read] = 0;
			bot.dataQ += read; /* compteur du traffic incoming*/

			while(*ptr)
			{/* si ircd compatible, \r avant \n..*/
				if(*ptr == '\n') 		/* fin de la ligne courante*/
				{
					tbuf[t-1] = 0;		/* efface le \r*/
					parse_this(tbuf);
					t = 0;				/* index du buffer permanent*/
				}
				else tbuf[t++] = *ptr;	/* copie dans le buffer permanent */
				ptr++;					/* qui garde la ligne courante*/
			}
		}
#ifdef TESTBOT
				else if(i == bind_sock)
				{
					int newfd;
					struct sockaddr_in remoteaddr;
					unsigned int addrlen = sizeof remoteaddr;
					char ip[16] = {0};
					w2c_hallow *hallow;

					if((newfd = accept(bind_sock, (struct sockaddr *) &remoteaddr, &addrlen)) == -1) continue;
					else
					{
						strcpy(ip, inet_ntoa(remoteaddr.sin_addr));
						for(hallow = w2c_hallowhead;hallow && match(hallow->host, ip);hallow = hallow->next);
						if(!hallow)
						{
							close(newfd);
							continue; /*non autoris*/
						}
						CONtotal++;
						FD_SET(newfd, &master);
						if(newfd > highsock) highsock = newfd;
						w2c_sendrpl(myWClient[newfd], RPL_000); /*Hello World!*/
					}
				}
				else
				{
					char buf[1024] = {0};

					if ((read = recv(i, buf, sizeof buf, 0)) <= 0) w2c_delclient(myWClient[i]);
					else
					{
						WEBtrafficDL += read; /* Faire une boucle de strtok comme pour le parse ircsock */
						w2c_parsemsg(myWClient[i], buf);
					}
				}

			} /* end for */
			FD_ZERO(&socks);
		} /* end check sock*/
#endif /*TESTBOT*/

	} /* end main loop */
	if (want_to_quit)
	{
		close(sock);
#ifdef TESTBOT
		w2c_closesocks();
#endif
	}
	bot.sock = 0;
	return 0;
}
